/* 
 * Crane XMMS Plugin
 * Special thanks to: 
 * http://www.xmms.org/docs/vis-plugin.html
 */
   
#include <stdio.h>
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>
#include <xmms/plugin.h>

/* The future Conan? */
static void init_crane_plug(void);
static void kill_crane_plug(void);
static void about_crane_plug(void);
static void conf_crane_plug(void);
static void start_crane_plug(void);
static void stop_crane_plug(void);
static void render_freq_crane_plug(gint16 freq[2][256]);

/* Global frequency info */
gint16 g_freq[2][256];


/* Info */
VisPlugin crane_plug = { 0, 0, 0,
                        "Ultimate Cranage", 0, 1,
                        init_crane_plug,
                        kill_crane_plug,
                        about_crane_plug,
                        conf_crane_plug,
                        0,
                        start_crane_plug,
                        stop_crane_plug,
                        0,
                        render_freq_crane_plug};


/* Threads */
SDL_Thread *thread;
SDL_mutex  *mutex;
enum {GO, STOP} thread_ctl;

void working_thread_proc(void)
{
    while(thread_ctl == GO)
    {
        SDL_mutexP(mutex);
        printf("Bass: %d\n", g_freq[0][0]);
        SDL_mutexV(mutex);
        xmms_usleep(1000000);
    }
}


/*************\
|* Callbacks *|
\*************/

/* XMMS needs some info */
VisPlugin *get_vplugin_info(void)
{
    return &crane_plug;
}


/* Initalize */
static void init_crane_plug(void)
{
    if(SDL_Init(SDL_INIT_VIDEO) < 0)
    {
        printf("Error initalizing SDL for threads.\n");
        crane_plug.disable_plugin(&crane_plug);
    }

    thread_ctl = GO;
    mutex = SDL_CreateMutex();
    memset(g_freq, 0, sizeof(gint16)*2*256);
    thread = SDL_CreateThread((void *)working_thread_proc, NULL);
}


/* Kill */
static void kill_crane_plug(void)
{
    printf("Croak...");
    thread_ctl = STOP;
    SDL_WaitThread(thread, NULL);
    SDL_DestroyMutex(mutex);
    SDL_Quit();
}


/* About */
static void about_crane_plug(void)
{
    printf("About...");
}


/* Configure */
static void conf_crane_plug(void)
{
    printf("Configure...");
}


/* Start */
static void start_crane_plug(void)
{
    printf("Start...");
}


/* Stop */
static void stop_crane_plug(void)
{
    printf("Stop...");
}


/* Deal with frequencies... dance */
static void render_freq_crane_plug(gint16 freq[2][256])
{
    /* Ok now we have the latest freq info, make it avaible
     * too all other funcs 
     */
     SDL_mutexP(mutex);
     memcpy(g_freq, freq, sizeof(gint16)*2*256);
     SDL_mutexV(mutex);
}
